#!/bin/bash

export NUMITER=10000
export STATICARRAYNUMITER=1000000
export BUCKETSORTNUMITER=10000000
export BUCKETSORTINTERNALNUMITER=10000000
export SORTLARGEEXISTENTIALSNUMITER=1000
#export SAMPLES=10
export SAMPLES=2
export TIMEFORMAT=%R
#export SWIFTC="`pwd`/../bin/swiftc"
export SWIFTC="/Users/rajbarik/workspace/swift-master-08052019/build/MyBuild/swift-macosx-x86_64/bin/swiftc"
#set -x
OUTPUT_FILE="table-4-perf.txt"
> $OUTPUT_FILE

execute() {
  if [ ! -f $1 ]; then
    echo "File not found!"
    exit
  fi
  local sum=0
  for i in $(seq 1 $SAMPLES)
  do
    #local new="$(time (./$1 $NUMITER 2> /dev/null 1>&2 ) 2>&1)"
    local ITER=$NUMITER
    if [ "$1" == "StaticArray" ]
    then 
      ITER=$STATICARRAYNUMITER
    elif [ "$1" == "BucketSort" ]
    then 
      ITER=$BUCKETSORTNUMITER
    elif [ "$1" == "BucketSortInternal" ]
    then 
      ITER=$BUCKETSORTINTERNALNUMITER
    elif [ "$1" == "SortLargeExistentials" ]
    then
      ITER=$SORTLARGEEXISTENTIALSNUMITER
    fi
    local new="$(./$1 $ITER|cut -d' ' -f4)"
    sum=`echo "$sum + $new" | bc`
  done
  local time=`echo "scale=4 ; $sum / $SAMPLES" | bc`
  echo "$time"
}

compileExecute() {
  local FILENAME="$1.swift"
  # Baseline compile
  #echo "Baseline $FILENAME"
  xcrun $SWIFTC $2 $FILENAME
  BASELINE="$(execute $1)"

  echo -n "Param "
  xcrun $SWIFTC $2 -Xfrontend -sil-existential-specializer $FILENAME 2>&1 > log.txt
  EXIST_PARAM="$(execute $1)"
  #echo $EXIST_PARAM
  EXIST_PARAM_RATIO=`echo "scale=2; ${BASELINE} / ${EXIST_PARAM}" | bc`
  echo $EXIST_PARAM_RATIO

  echo -n "SoleType "
  xcrun $SWIFTC $2 -Xfrontend -sil-sole-type-propagation $FILENAME 2>&1 > log.txt
  EXIST_SOLE="$(execute $1)"
  #echo $EXIST_SOLE
  EXIST_SOLE_RATIO=`echo "scale=2; ${BASELINE} / ${EXIST_SOLE}" | bc`
  echo $EXIST_SOLE_RATIO

  echo -n "LocalVar "
  xcrun $SWIFTC $2 -Xfrontend -sil-existential-value-specializer $FILENAME 2>&1 > log.txt
  EXIST_VALUE="$(execute $1)"
  #echo $EXIST_VALUE
  EXIST_VALUE_RATIO=`echo "scale=2; ${BASELINE} / ${EXIST_VALUE}" | bc`
  echo $EXIST_VALUE_RATIO

  echo -n "Param+SoleType "
  xcrun $SWIFTC $2 -Xfrontend -sil-existential-specializer -Xfrontend -sil-sole-type-propagation $FILENAME 2>&1 > log.txt
  EXIST_PARAM_SOLE="$(execute $1)"
  #echo $EXIST_PARAM_SOLE
  EXIST_PARAM_SOLE_RATIO=`echo "scale=2; ${BASELINE} / ${EXIST_PARAM_SOLE}" | bc`
  echo $EXIST_PARAM_SOLE_RATIO

  echo -n "Param+LocalVar "
  xcrun $SWIFTC $2 -Xfrontend -sil-existential-specializer -Xfrontend -sil-existential-value-specializer $FILENAME 2>&1 > log.txt
  EXIST_PARAM_VALUE="$(execute $1)"
  #echo $EXIST_PARAM_VALUE
  EXIST_PARAM_VALUE_RATIO=`echo "scale=2; ${BASELINE} / ${EXIST_PARAM_VALUE}" | bc`
  echo $EXIST_PARAM_VALUE_RATIO

  echo -n "SoleType+LocalVar "
  xcrun $SWIFTC $2 -Xfrontend -sil-sole-type-propagation -Xfrontend -sil-existential-value-specializer $FILENAME 2>&1 > log.txt
  EXIST_SOLE_VALUE="$(execute $1)"
  #echo $EXIST_SOLE_VALUE
  EXIST_SOLE_VALUE_RATIO=`echo "scale=2; ${BASELINE} / ${EXIST_SOLE_VALUE}" | bc`
  echo $EXIST_SOLE_VALUE_RATIO

  echo -n "Param+SoleType+LocalVar "
  xcrun $SWIFTC $2 -Xfrontend -sil-existential-specializer -Xfrontend -sil-sole-type-propagation -Xfrontend -sil-existential-value-specializer $FILENAME 2>&1 > log.txt
  EXIST_ALL="$(execute $1)"
  #echo $EXIST_ALL
  EXIST_ALL_RATIO=`echo "scale=2; ${BASELINE} / ${EXIST_ALL}" | bc`
  echo $EXIST_ALL_RATIO

  #echo $1, $2, $BASELINE, $EXIST_PARAM, $EXIST_SOLE, $EXIST_VALUE, $EXIST_PARAM_SOLE, $EXIST_PARAM_VALUE, $EXIST_SOLE_VALUE, $EXIST_ALL
  echo $1, $EXIST_PARAM_RATIO, $EXIST_SOLE_RATIO, $EXIST_VALUE_RATIO, $EXIST_PARAM_SOLE_RATIO, $EXIST_PARAM_VALUE_RATIO, $EXIST_SOLE_VALUE_RATIO, $EXIST_ALL_RATIO
  echo $1, $EXIST_PARAM_RATIO, $EXIST_SOLE_RATIO, $EXIST_VALUE_RATIO, $EXIST_PARAM_SOLE_RATIO, $EXIST_PARAM_VALUE_RATIO, $EXIST_SOLE_VALUE_RATIO, $EXIST_ALL_RATIO >> $OUTPUT_FILE
}

#export SDKROOT="/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.14.sdk"
export SDKROOT="/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.15.sdk"

declare -a files=("ArrayOfGenericRef" "ArrayOfRef" "Codable" "DictionarySubscriptDefault" "NSError" "ObserverForwarderStruct" "ObserverUnappliedMethod" "PopFrontGeneric" "ProtocolDispatch2" "SortLargeExistentials" "StackPromo" "StaticArray" "BucketSort")
echo "Benchmark | Param | SoleType | LocalVar | Param+SoleType | Param+LocalVar | SoleType+LocalVar | Param+SoleType+LocalVar"
for i in "${files[@]}"
do
compileExecute $i "-O -wmo"
done
